// file      : structure-xerces.cxx
// author    : Boris Kolpackov <boris@codesynthesis.com>
// copyright : Copyright (c) 2006 Code Synthesis Tools CC
// license   : GNU GPL v2; see accompanying LICENSE file

#include <iostream>
#include <fstream>

#include <xercesc/sax2/SAX2XMLReader.hpp>
#include <xercesc/sax2/DefaultHandler.hpp>
#include <xercesc/sax2/XMLReaderFactory.hpp>

#include <xercesc/framework/MemBufInputSource.hpp>

#include <xercesc/util/XMLString.hpp>

#include "time.hxx"

using namespace std;
using namespace xercesc;

struct Handler: DefaultHandler
{
  virtual void
  startElement(const XMLCh* const,
               const XMLCh* const,
               const XMLCh* const,
               const Attributes&)
  {
  }

  virtual void
  endElement(const XMLCh* const,
             const XMLCh* const,
             const XMLCh* const)
  {
  }

  virtual void
  characters (const XMLCh* const conss, const unsigned int)
  {
  }

  virtual void
  error (const xercesc::SAXParseException& e)
  {
    fatalError (e);
  }

  virtual void
  fatalError (const xercesc::SAXParseException& e)
  {
    const XMLCh* id (e.getPublicId () ? e.getPublicId () : e.getSystemId ());

    XMLSSize_t l (e.getLineNumber ());
    XMLSSize_t c (e.getColumnNumber ());

    // We are leaking memory here.
    //
    cerr << XMLString::transcode (id) << ":" << l << ":" << c << ": "
         << XMLString::transcode (e.getMessage ()) << endl;

    exit (1);
  }
};

int
main (int argc, char* argv[])
{
  if (argc != 2)
  {
    cerr << "usage: " << argv[0] << " strcuture.xml" << endl;
    return 1;
  }

  try
  {
    ifstream ifs;
    ifs.exceptions (ios_base::failbit);
    ifs.open (argv[1], ios::in | ios::ate);

    size_t size (ifs.tellg ());
    ifs.seekg (0, ios::beg);

    char* buf = new char[size];
    ifs.read (buf, size);

    cerr << "document size: " << size << " bytes" << endl;

    XMLPlatformUtils::Initialize ();

    {
      SAX2XMLReader* sax (XMLReaderFactory::createXMLReader());

      sax->setFeature (XMLUni::fgSAX2CoreNameSpaces, true);
      sax->setFeature (XMLUni::fgSAX2CoreNameSpacePrefixes, true);
      sax->setFeature (XMLUni::fgXercesValidationErrorAsFatal, true);
      sax->setFeature (XMLUni::fgSAX2CoreValidation, true);
      sax->setFeature (XMLUni::fgXercesSchema, true);
      sax->setFeature (XMLUni::fgXercesSchemaFullChecking, true);

      //sax->setFeature (XMLUni::fgXercesCacheGrammarFromParse, true);

      sax->loadGrammar ("structure.xsd", Grammar::SchemaGrammarType, true);
      sax->setFeature (XMLUni::fgXercesUseCachedGrammarInParse, true);

      Handler handler;
      sax->setContentHandler (&handler);
      sax->setErrorHandler (&handler);

      MemBufInputSource is (reinterpret_cast<XMLByte*> (buf),
                            size,
                            "structure.xml",
                            false);

      is.setCopyBufToStream (false);

      OS::Time start;

      for (unsigned long i (0); i < 10000; ++i)
      {
        sax->parse (is);
      }

      OS::Time end;

      cerr << "time: " << end - start << endl;

      delete sax;
    }

    XMLPlatformUtils::Terminate ();

    delete[] buf;
  }
  catch (std::ios_base::failure const&)
  {
    cerr << "io failure" << endl;
    return 1;
  }
}
